/* 
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/
/**
   Reading an integer in the buffer assigned to a file register (filid).

   @return false if eof is detected on filid before reading, true otherwise (val is available).
   @param filid (in) : a file register.
   @param val (out) the current integer of filid and forwards.
*/
extern boolean
sysgetint (ptrepfil filid, int *val)
{
  if (filid == 3)
    return getint (&if3, val);
  if (filid == 4)
    return getint (&if4, val);
  if (filid == 5)
    return getint (&if5, val);
  if (filid == 6)
    return getint (&if6, val);
  printf (" Sysfile module, function sysgetint : unknown file identifier.\n");
  return pk_false;
}


/**
   Reading a line in the buffer assigned to a file register (filid) from the 
   opened file assigned to the same file register (filid).

   @return false if eof is detected on filid before reading, true otherwise.
   @param filid (in) : a file register.
*/
extern boolean
sysnewline (ptrepfil filid)
{
  if (filid == 3)
    return newline (&if3);
  if (filid == 4)
    return newline (&if4);
  if (filid == 5)
    return newline (&if5);
  if (filid == 6)
    return newline (&if6);
  printf (" Sysfile module, function sysnewline : unknown file identifier.\n");
  return pk_false;
}


/**
   Forwarding to a specified character (c) in the buffer assigned to a file register (filid).
   The current character is considered.
   @return true if the character is found.
   \note this function is only used in the tabtools.c module.
*/
extern boolean
syssktoeq (ptrepfil filid, char c)
{
  if (filid == 3)
    return sktoeq (&if3, c);
  if (filid == 4)
    return sktoeq (&if4, c);
  if (filid == 5)
    return sktoeq (&if5, c);
  if (filid == 6)
    return sktoeq (&if6, c);
  printf (" Sysfile module, function syssktoeq : unknown file identifier.\n");
  return pk_false;
}


/**
   Forwarding to the next character different of c in the buffer assigned to a file register (filid).
   The current character is tested.
   @return  "true" if an occurrence different of c is found, "false" otherwise or the file identifier does not exist.
*/
extern boolean
syssktone (ptrepfil filid, char c)
{
  if (filid == 3)
    return sktone (&if3, c);
  if (filid == 4)
    return sktone (&if4, c);
  if (filid == 5)
    return sktone (&if5, c);
  if (filid == 6)
    return sktone (&if6, c);
  printf (" Sysfile module, function syssktone :  unknown file identifier.\n");
  return pk_false;
}

/**
   Opening a file using a file register.

   Opens the file name in the "write" mode if w and in the "read" mode otherwise.
   @return  a file register descriptor when the operation succeeds, or NILFIL otherwise.

   \warning Only 4 files can be simultaneously opened.
*/
extern ptrepfil
sysopen (ptrepstr name, boolean w)
{
  if (filreg3.name == nilstr)
    {
      filreg3.name = name;
      syslocopen (&if3, name, w);
      return 3;
    }
  if (filreg4.name == nilstr)
    {
      filreg4.name = name;
      syslocopen (&if4, name, w);
      return 4;
    }
  if (filreg5.name == nilstr)
    {
      filreg5.name = name;
      syslocopen (&if5, name, w);
      return 5;
    }
  if (filreg6.name == nilstr)
    {
      filreg6.name = name;
      syslocopen (&if6, name, w);
      return 6;
    }
  printf (" Sysfile module, function sysopen : cannot opening a file using a file register.\n");
  return NILFIL;
}

/**
   Opening the standard output using a file register.
   @param code : codification of the output :
             - 0 for screen output,
             - 1 or 2 graphical output.
   @return  the selected file register according to the code value.
*/
extern ptrepfil
sysopenttyout (int code)
{
  switch (code)
    {
    case 0:
      return outfil;
    case 1:
      return ttygraph;
    case 2:
      return outgraph;
    }
  return NILFIL;
}



/**
   Assigns a buffer (fbuffer) to the file register (filid).
   \note This function is called only in the function bufopen defined in the BUFTOOLS module .
*/
extern void
sysbuflink (ptrepfil filid, ptreg fbuffer)
{
  if (filid == 3)
    filreg3.buffer = fbuffer;
  else if (filid == 4)
    filreg4.buffer = fbuffer;
  else if (filid == 5)
    filreg5.buffer = fbuffer;
  else if (filid == 6)
    filreg6.buffer = fbuffer;
  else
    printf (" Sysfile module, function sysbuflink : unknown file identifier.\n");
}



/**
   Closing the file register (filid) and freeing the assigned "switch".
   @return "true" if the operation succeeds,"false" otherwise (unknown filid).
*/
extern boolean
sysclose (ptrepfil filid)
{
  if (filid == 3)
    {
      filreg3.name = nilstr;
      Fclose (if3);
      return pk_true;
    }
  if (filid == 4)
    {
      filreg4.name = nilstr;
      Fclose (if4);
      return pk_true;
    }
  if (filid == 5)
    {
      filreg5.name = nilstr;
      Fclose (if5);
      return pk_true;
    }
  if (filid == 6)
    {
      filreg6.name = nilstr;
      Fclose (if6);
      return pk_true;
    }
  printf (" Sysfile module, function sysclose : unknown file identifier.\n");
  return pk_false;
}



/** 
    Closing the standard output.
*/
extern void
sysclosettyout (ptrepfil /* filid */ )
{
}

/** 
    Testing EOF on a file register (filid).
    @return true if EOF is detected.
*/
extern boolean
syseof (ptrepfil filid)
{
  if (filid == 3)
    return Eof (if3);
  if (filid == 4)
    return Eof (if4);
  if (filid == 5)
    return Eof (if5);
  if (filid == 6)
    return Eof (if6);
  printf (" Sysfile module, function syseof : unknown file identifier.\n");
  return pk_false;
}

/**
   Testing EOL on a file register (filid).
   @return true if EOL is detected.
*/
extern boolean
syseoline (ptrepfil filid)
{
  if (filid == 3)
    return EoLine (if3);
  if (filid == 4)
    return EoLine (if4);
  if (filid == 5)
    return EoLine (if5);
  if (filid == 6)
    return EoLine (if6);
  printf (" Sysfile module, function syseoline : unknown file identifier.\n");
  return pk_true;
}


/**
   Reading the current character of the buffer assigned to a file register (filid).
   The charqcter is returned in cc.
   @return  false if EOF is detected, true otherwise.
*/
extern boolean
sysgetcha (ptrepfil filid, char *cc)
{
  if (filid == 3)
    return getcha (&if3, cc);
  if (filid == 4)
    return getcha (&if4, cc);
  if (filid == 5)
    return getcha (&if5, cc);
  if (filid == 6)
    return getcha (&if6, cc);
  printf (" Sysfile module, function sysgetcha : unknown file identifier.\n");
  return pk_true;
}




/**
   Catenating in the buffer assigned to a file register filid the set of characters from current position 
   to the character c not included. It forwards after the character c.

   @return  false if the buffer is filled before finding c.
   \note This function is called only in the TABTOOLS module.
*/
extern boolean
sysrdtoeq (ptrepfil filid, char c)
{
  if (filid == 3)
    return rdtoeq (filreg3, &if3, c);
  if (filid == 4)
    return rdtoeq (filreg4, &if4, c);
  if (filid == 5)
    return rdtoeq (filreg5, &if5, c);
  if (filid == 6)
    return rdtoeq (filreg6, &if6, c);
  printf (" Sysfile module, function sysrdtoeq : unknown file identifier.\n");
  return pk_true;
}



/**
   Writing a character (c) in the buffer assigned to a file register (filid).
*/
extern void
sysprcha (ptrepfil filid, char c)
{
  if (filid == 3)
    Putchr (c, if3);
  else if (filid == 4)
    Putchr (c, if4);
  else if (filid == 5)
    Putchr (c, if5);
  else if (filid == 6)
    Putchr (c, if6);
  else
    printf (" Sysfile module, function sysprcha : unknown file identifier.\n");
}



/**
   Writing l times a character (c) in the buffer assigned to a file register (filid).
*/
extern void
sysprmcha (ptrepfil filid, char c, int l)
{
  int i;
  text aFile;

  if (l > 0)
    {
      if (filid == 3)
	aFile = if3;
      else if (filid == 4)
	aFile = if4;
      else if (filid == 5)
	aFile = if5;
      else if (filid == 6)
	aFile = if6;
      else
	{
	  printf (" Sysfile module, function sysprmcha : unknown file identifier.\n");
	  return;
	}

      for (i = 1; i <= l; i++)
	Putchr (c, aFile);
    }
}


/**
   Writing a register (str) in the buffer assigned to a file register (filid) 
   and finalyzing the line.
*/
extern void
syswriteln (ptrepfil filid, ptreg str)
{
  if (filid == 3)
    locwriteln (filreg3, &if3, str);
  else if (filid == 4)
    locwriteln (filreg4, &if4, str);
  else if (filid == 5)
    locwriteln (filreg5, &if5, str);
  else if (filid == 6)
    locwriteln (filreg6, &if6, str);
  else
    printf (" Sysfile module, function syswriteln : unknown file identifier.\n");

}
